#! /usr/bin/perl
#! /sw/perl-5.005/bin/perl
require 5.005;
use strict;
use DBI;
use POSIX qw(:sys_wait_h :time_h);

use File::Basename;
use lib dirname($0);
use Scripts qw(parseQueryString @mon);
#use OraWrapper;
use PgWrapper;

#my $gnuplot = '/sw/gnuplot/bin/gnuplot';
my $gnuplot = '/opt/gnuplot/bin/gnuplot';
my $tmpdir = $ENV{TMP} || '/tmp';

BEGIN {
    umask(022);
}

sub numerical ($) {
    # purpose: obtain single numerical parameter and untaint it
    my $x = shift;
    $x = $x->[ @{$x}-1 ] if ref $x;
    $x =~ tr/[0-9]//cs;
    $x+0;
}

# sanitize, see CERT warning
my %form = parseQueryString();
my $shour = numerical($form{shour});
my $syear = numerical($form{syear});
$syear -= 1900 if $syear > 1900;
my $sday  = numerical($form{sday});
my $smon  = numerical($form{smon});
my $start = mktime(0,0,$shour,$sday,$smon,$syear);

# parse finish time
my $fhour = numerical($form{fhour});
my $fyear = numerical($form{fyear});
$fyear -= 1900 if $fyear > 1900;
my $fday  = numerical($form{fday});
my $fmon  = numerical($form{fmon});
my $final = mktime(0,0,$fhour,$fday,$fmon,$fyear);

if ( ! exists $form{cache} ) {
    print << "EOF";
Content-Type: text/plain; charset=iso-8859-1
Cache-Control: private

Sorry, you have to activate at least one cache host.
EOF
    exit(1);
} 
my @cache = ref $form{cache} ? @{$form{cache}} : ($form{cache});
my $n = 0+@cache;


my $type = numerical($form{type});
my @title = ( 'hourly TCP and TCP HIT',
	      'non GET methods',
	      '*.win-ip.dfn.de as TCP client',
	      'IN, OUT and AMONG'
	      );

my $kind = numerical($form{kind});
my @yaxis = ( 'Requests [1]', 
	      'Volume [Byte]' );

my @isize = ( '640,480', '800,600', '1024,768' );
my $isize = numerical($form{isize});
my $format = numerical($form{format});
my @suffix = qw(.gif .png .eps);
my @ctype = ( 'image/gif', 
	      'image/png',
	      'application/postscript'
	      );
my $pid = $$;
my @result;
my $cf = "$tmpdir/$pid.cf";
my $of = "$cf.$suffix[$format]";
my @callya = ( \&ctor0, \&ctor1, \&ctor2, \&ctor3 );

eval {
    open( CF, ">$cf" ) || die "open($cf): $!";
    push @Apache::Registry::filename, $cf; # unlink after exit
    if ( $format == 0 ) {
	# gif output
	print CF "set term gif interlace small size $isize[$isize]\n";
    } elsif ( $format == 1 ) {
	print CF "set term png small color\n";
	print CF "set size 1.44,1.44\n" if ( $isize == 1 );
	print CF "set size 2,2\n" if ( $isize == 2 );
    } else {
	print CF "set term post eps color solid \"Helvetica-Bold\" ";
	if ( $isize == 0 ) {
	    print CF "14\nset size 1.4,1.4\n";
	} elsif ( $isize == 1 ) {
	    print CF "18\nset size 2,2\n";
	} else {
	    print CF "24\nset size 3,3\n";
	}
    }
    print CF << "EOF";
set output "$of"
set grid
set xdata time
set timefmt "%s"
set format x "%d.%m"
set title "$title[$type]"
set ylabel "$yaxis[$kind]"
EOF
    print CF "plot ";
    if ( defined $callya[$type] ) {
	no strict 'subs';
	&{$callya[$type]}(CF);
    } else {
	die "Sorry, you selected the query of type=$type" .
	    " which is not implemented.\n";
    }
    close(CF);
};

# check for errors during db connection, cluck and exit
if ( $@ ne '' ) {
    print << "EOF";
Content-Type: text/plain; charset=iso-8859-1
Cache-Control: private

Error during execution of database connection:

$@
EOF
    exit(1);
}

system("$gnuplot $cf");
my $result=0+$?;
my $rsize = -s "$of";
print "Content-Type: $ctype[$format]\r\n";
print "Content-Length: ", 0+$rsize, "\r\n";
print "Cache-Control: max-age=5\r\n\r\n";

open( IN, "<$of" );
my $buffer;
syswrite(STDOUT,$buffer,$rsize) while ( ($rsize=read(IN,$buffer,4096)) > 0 );
close(IN);
unlink "$of";

#
# ---------------------------------------------------------------
#
sub ctor0 (*) {
    local *CF = shift;
    for ( my $i=0; $i<$n; ++$i ) {
	my $fn = "$tmpdir/$pid.$i";
	open( OUT, ">$fn" ) || die "open($fn): $!";
	push @Apache::Registry::filename, $fn; # unlink after exit

	my $q = 'select st.first, ';
	$q .= $kind ? 'p.tcp_s, p.tcp_hit_s ' : 'p.tcp_r, p.tcp_hit_r ' ;
	$q .= 'from sf_stamp2 st, sf_peak p ' .
	    "where st.first between $start and $final and " .
	    "st.i2id=p.i2id and st.cache=\'$cache[$i]\' " .
	    'order by st.first';
	my $qh = $Apache::Registry::sf->prepare($q) or
	    die "cannot prepare($q): $DBI::errstr";
	my $rc = $qh->execute or
	    die "cannot execute($q): $DBI::errstr";

	my @row;
	print OUT join("\t",@row), "\n"
	    while ( @row = $qh->fetchrow_array );
	$qh->finish;
	undef @row;
	undef $qh;
	undef $q;

	close(OUT);
	print CF ",\\\n " if ( $i );
	print CF "\"$fn\" using 1:2 title \"", $cache[$i], "\" w l, \\\n ";
	print CF "\"$fn\" using 1:3 title \"H-", $cache[$i], "\" w l";
    }
}

#
# ---------------------------------------------------------------
#
sub ctor1 (*) {
    local *CF = shift;
    for ( my $i=0; $i<$n; ++$i ) {
	my $fn = "$tmpdir/$pid.$i";
	open( OUT, ">$fn" ) || die "open($fn): $!";
	push @Apache::Registry::filename, $fn; # unlink after exit

	my $q = 'select st.first, ';
	$q .= $kind ? 'sum(m.s), sum(m.hits) ' : 'sum(m.r), sum(m.hitr) ';
	$q .= 'from sf_stamp1 st, sf_method m ' .
	      "where st.first between $start and $final and " .
	      "st.i1id=m.i1id and st.cache=\'$cache[$i]\' " .
	      'group by st.first ' .
	      'order by st.first';
	my $qh = $Apache::Registry::sf->prepare($q) or
	    die "cannot prepare($q): $DBI::errstr";
	my $rc = $qh->execute or
	    die "cannot execute($q): $DBI::errstr";

	my @row;
	print OUT join("\t",@row), "\n" while ( @row = $qh->fetchrow_array );
	$qh->finish;
	undef @row;
	undef $qh;
	undef $q;

	close(OUT);
	print CF ",\\\n " if ( $i );
	print CF "\"$fn\" using 1:2 title \"", $cache[$i], "\" w l, \\\n ";
	print CF "\"$fn\" using 1:3 title \"H-", $cache[$i], "\" w l";
    }
}

#
# ---------------------------------------------------------------
#
sub ctor2 (*) {
    local *CF = shift;
    for ( my $i=0; $i<$n; ++$i ) {
	my $fn = "$tmpdir/$pid.$i";
	open( OUT, ">$fn" ) || die "open($fn): $!";
	push @Apache::Registry::filename, $fn; # unlink after exit

	my $q = 'select st.first, ';
	$q .= $kind ? 'sum(c.s) ' :	'sum(c.r) ';
	$q .= 'from sf_stamp1 st, sf_tcp_client c ' .
	      "where st.first between $start and $final and " .
	      "st.i1id=c.i1id and st.cache=\'$cache[$i]\' and " .
	      "(c.host like \'%.win-ip.dfn.de\' or c.host like \'193.174.75.%\') " .
	      'group by st.first order by st.first';
	my $qh = $Apache::Registry::sf->prepare($q) or
	    die "cannot prepare($q): $DBI::errstr";
	my $rc = $qh->execute or
	    die "cannot execute($q): $DBI::errstr";

	my @row;
	print OUT join("\t",@row), "\n"	while ( @row = $qh->fetchrow_array );
	$qh->finish;
	undef @row;
	undef $qh;
	undef $q;

	close(OUT);
	print CF ",\\\n " if ( $i );
	print CF "\"$fn\" using 1:2 title \"", $cache[$i], "\" w l";
    }
}

#
# ---------------------------------------------------------------
#
sub ctor3 (*) {
    local *CF = shift;
    my $cacheset;
    for ( my $i=0; $i<$n; ++$i ) {
	$cacheset .= ',' if $i;
	$cacheset .= "\'$cache[$i]\'";
    }
    
    # OUT from tcp_client w/o dfn caches
    my $q = 'select st.first, ';
    $q .= $kind ? 'sum(tc.s) ' : 'sum(tc.r) ';
    $q .= 'from sf_stamp1 st, sf_tcp_client tc ';
    $q .= "where st.i1id=tc.i1id and st.cache in ( $cacheset ) ";
    $q .= "and st.first between $start and $final ";
    $q .= "and not ( tc.host like \'%.win-ip.dfn.de\' ";
    $q .= "or tc.host like \'193.174.75.%\' ) ";
    $q .= 'group by st.first';

    my $qh = $Apache::Registry::sf->prepare($q) or
	die "cannot prepare($q): $DBI::errstr";
    my $rc = $qh->execute or
	die "cannot execute($q): $DBI::errstr";

    my %benefit;
    my @row;
    $benefit{$row[0]}[0] = $row[1] while ( @row = $qh->fetchrow_array );
    $qh->finish;
    undef @row;
    undef $qh;
    undef $q;

    # IN from sf_hier_direct
    $q = 'select st.first, ';
    $q .= $kind ? 'sum(hd.s) ' : 'sum(hd.r) ';
    $q .= 'from sf_stamp1 st, sf_hier_direct hd ';
    $q .= "where st.i1id=hd.i1id and st.cache in ( $cacheset ) ";
    $q .= "and st.first between $start and $final ";
    $q .= 'group by st.first';

    $qh = $Apache::Registry::sf->prepare($q) or
	die "cannot prepare($q): $DBI::errstr";
    $rc = $qh->execute or
	die "cannot execute($q): $DBI::errstr";

    $benefit{$row[0]}[1] = $row[1] while ( @row = $qh->fetchrow_array );
    $qh->finish;
    undef @row;
    undef $qh;
    undef $q;

    # AMONG from hier_peer (siblings and parents)
    $q = 'select st.first, ';
    $q .= $kind ? 'sum(hp.s) ' : 'sum(hp.r) ';
    $q .= 'from sf_stamp1 st, sf_hier_peer hp where st.i1id=hp.i1id ';
    $q .= "and st.cache in ( $cacheset ) ";
    $q .= "and st.first between $start and $final ";
    $q .= 'group by st.first';

    $qh = $Apache::Registry::sf->prepare($q) or
	die "cannot prepare($q): $DBI::errstr";
    $rc = $qh->execute or
	die "cannot execute($q): $DBI::errstr";

    $benefit{$row[0]}[3] = $row[1] while ( @row = $qh->fetchrow_array );
    $qh->finish;
    undef @row;
    undef $qh;
    undef $q;

    # SAVINGS as OUT - IN
    my $key;
    foreach $key ( keys %benefit ) {
	$benefit{$key}[2] = $benefit{$key}[0] - $benefit{$key}[1]
	    if ( defined $benefit{$key}[0] && defined $benefit{$key}[1] );
    }

    # create output
    my $fn = "$tmpdir/$pid.0";
    open( OUT, ">$fn" ) || die "open($fn): $!";
    push @Apache::Registry::filename, $fn; # unlink after exit

    foreach $key ( sort keys %benefit ) {
	print OUT "$key ", join(' ', map { $_+0 } @{$benefit{$key}} );
    }

    close(OUT);
    print CF "\"$fn\" using 1:2 title \"OUT\" w l";
    print CF ",\\\n \"$fn\" using 1:3 title \"IN\" w l";
    print CF ",\\\n \"$fn\" using 1:3 title \"SAVINGS\" w l";
    print CF ",\\\n \"$fn\" using 1:3 title \"AMONG\" w l";
}
