#! /usr/bin/perl
#! /sw/perl-5.005/bin/perl
require 5.005;
use strict;
use DBI;

use File::Basename;
use lib dirname($0);
use Scripts qw(@mon);
#use OraWrapper;
use PgWrapper;

# use eval() for database connection in order to die gracefully on errors
my ($start,$final,@cache);
eval {
    my $q = 'select min(first),max(first) from sf_stamp1';
    my $qh = $Apache::Registry::sf->prepare($q) or
	die "cannot prepare($q): ", $DBI::errstr;
    my $rc = $qh->execute or
	die "cannot execute($q): ", $DBI::errstr;
       ($start,$final) = $qh->fetchrow_array;
    $qh->finish;

    $q = "select distinct cache from sf_stamp1 order by cache";
    $qh = $Apache::Registry::sf->prepare($q) or
        die "cannot prepare($q): ", $Apache::Registry::sf->errstr;
    my $rc = $qh->execute or
        die "cannot execute($q): ", $Apache::Registry::sf->errstr;
    my @row;
    push @cache, $row[0] while ( @row = $qh->fetchrow_array );
    undef @row;
    $qh->finish;
};

# check for errors during db connection, cluck and exit
if ( $@ ne '' ) {
    print << "EOF";
Content-Type: text/plain; charset=iso-8859-1
Cache-Control: private; max-age=0

Error during execution of database connection:

$@
EOF
    exit(1);
}

# convert UTC stamps into local time
my @start = localtime($start);
my @final = localtime($final);

sub create (\@$$$) {
    my ($array,$start,$stop,$current) = @_;
    for ( my $i=$start; $i<$stop; ++$i ) {
        push @{$array}, sprintf( "<OPTION%s>%02d</OPTION>\n",
                                 $i == $current ? " SELECTED" : "", $i );
    }
}

sub create_mon (\@$$$) {
    my ($array,$start,$stop,$current) = @_;
    for ( my $i=$start; $i<$stop; ++$i ) {
	push @{$array}, sprintf( "<OPTION%s VALUE=\"%d\">%3s</OPTION>\n",
				 $i == $current ? " SELECTED" : "", $i,
				 $mon[$i] );
    }
}

my (@shour,@sday,@smon,@syear);
create( @shour, 0, 24, $start[2] );
create( @sday, 1, 32, $start[3] );
create_mon( @smon, 0, 12, $start[4] );
for ( my $i = 1997; $i < 2010; ++$i ) {
    push @syear, sprintf( "<OPTION%s>%04d</OPTION>\n",
                           $i == 1900+$start[5] ? " SELECTED" : "",
                           $i );
}

my (@fhour,@fday,@fmon,@fyear);
create( @fhour, 0, 24, $final[2] );
create( @fday, 1, 32, $final[3] );
create_mon( @fmon, 0, 12, $final[4] );
for ( my $i = 1997; $i < 2010; ++$i ) {
    push @fyear, sprintf( "<OPTION%s>%04d</OPTION>\n",
                           $i == 1900+$final[5] ? " SELECTED" : "",
                           $i );
}
@cache = map { "<OPTION>$_</OPTION>\n" } @cache;

# create input form for next stage
print << "EOF";
Content-Type: text/html; charset=iso-8859-1
Cache-Control: public; max-age=86400

<HTML>
<HEAD>
<TITLE>Selection</TITLE>
</HEAD>
<BODY BGCOLOR="white">
<H1>Selection</H1>

The smallest time stamp in the database is $start aka
@{[scalar localtime($start)]},<BR>
and the largest time stamp is $final aka
@{[scalar localtime($final)]}.

<FORM ACTION="ctor.cgi" METHOD="GET">
<TABLE BORDER="0" BGCOLOR="#CCCCFF" CELLSPACING="0" CELLPADDING="2">
<TR>
 <TH ALIGN="right">Start Time:</TH>
 <TD><SELECT NAME="shour" SIZE="1">@shour
     </SELECT>hours, </TD>
 <TD><SELECT NAME="sday" SIZE="1">@sday
     </SELECT>.</TD>
 <TD><SELECT NAME="smon" SIZE="1">@smon
     </SELECT>.</TD>
 <TD><SELECT NAME="syear" SIZE="1">@syear
     </SELECT></TD>
</TR>
<TR>&nbsp;</TR>
<TR>
 <TH ALIGN="right">Final Time:</TH>
 <TD><SELECT NAME="fhour" SIZE="1">@fhour
     </SELECT>hours, </TD>
 <TD><SELECT NAME="fday" SIZE="1">@fday
     </SELECT>.</TD>
 <TD><SELECT NAME="fmon" SIZE="1">@fmon
     </SELECT>.</TD>
 <TD><SELECT NAME="fyear" SIZE="1">@fyear
     </SELECT></TD>
</TR>
<TR>&nbsp;</TR>
<TR>
 <TH ALIGN="right">Hosts:</TH>
 <TD COLSPAN="4"><SELECT NAME="cache" SIZE="@{[0+@cache]}" MULTIPLE>@cache
     </SELECT></TD>
</TR>
<TR>&nbsp;</TR>
<TR>
 <TH ALIGN="right">Diagram:</TH>
 <TD COLSPAN="2">
  <INPUT TYPE="radio" NAME="isize" VALUE="0"> small<BR>
  <INPUT TYPE="radio" CHECKED NAME="isize" VALUE="1"> medium<BR>
  <INPUT TYPE="radio" NAME="isize" VALUE="2"> large<BR>
 </TD>
 <TD COLSPAN="2">
   <INPUT TYPE="radio" CHECKED NAME="format" VALUE="0"> GIF</BR>
   <INPUT TYPE="radio" NAME="format" VALUE="1"> PNG</BR>
   <INPUT TYPE="radio" NAME="format" VALUE="2"> EPS</BR>
 </TD>
</TR>
<TR>&nbsp;</TR>
<TR>
 <TH ALIGN="right">Query:</TH>
 <TD COLSPAN="3"><SELECT NAME="type" SIZE="1">
    <OPTION VALUE="0" SELECTED>Peak TCP and TCP-HIT</OPTION>
    <OPTION VALUE="1">non GET methods</OPTION>
    <OPTION VALUE="2">*.win-ip.dfn.de TCP clients</OPTION>
    <OPTION VALUE="3">IN, OUT and AMONG</OPTION>
     </SELECT></TD>
 <TD>
    <INPUT TYPE="radio" CHECKED NAME="kind" VALUE="0"> by req.</BR>
    <INPUT TYPE="radio" NAME="kind" VALUE="1"> by vol.</BR>
 </TD>
</TR>
<TR>&nbsp;</TR>
<TR>
 <TH ALIGN="right">Action:</TH>
 <TD COLSPAN="2"><INPUT TYPE="submit" VALUE="continue"></TD>
 <TD COLSPAN="2"><INPUT TYPE="reset" VALUE="reset"></TD>
</TR>
</TABLE>
</FORM>
<P>

<H1>Some Explanations</H1>

<H3>Peak TCP and TCP-HIT</H3>

The peak performance is a simple query into the perfomance table
of the database. Please note that for every host, two graphs will
be displayed.

<H3>non-GET methods</H3>

The HTTP method GET is major part of all possible methods. This 
slightly more complex query displays two graphs for each selected
host, one with the !GET queries, and one with the HITs in those.
An improved version might show the percentage of !GET methods with
regards to all methods.

<H3>*.win-ip.dfn.de TCP clients</H3>

Hosts in the domain win-ip.dfn.de constitute the correct reverse
resolution of our top level caches. For each selected cache host,
the graph will show the sum of other nine caches querying the selected
cache.

<H3>IN, OUT and AMONG</H3>

The basic data to evaluate the efficiency of a cache are of two kinds:
the savings OUT minus IN, and the overhead AMONG. This time, the selected
hosts are put into one query group, for which the amount IN, OUT and
AMONG is determined.

<BLOCKQUOTE><IMG SRC="balance.gif" ALT="balance"></BLOCKQUOTE><P>

The information for IN is determined from all hierarchy types DIRECT, 
the value for AMONG from the sum of hierarchy types PEER and PARENT, and
the information for OUT is determined from the TCP client list minus
any DFN cache traffic.

</BODY>
</HTML>
EOF
